package org.dromara.service;

import org.dromara.domain.vo.TblBreakdownVo;
import org.dromara.domain.bo.TblBreakdownBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 故障管理Service接口
 *
 * @author Lion Li
 * @date 2024-08-11
 */
public interface ITblBreakdownService {

    /**
     * 查询故障管理
     *
     * @param id 主键
     * @return 故障管理
     */
    TblBreakdownVo queryById(Long id);

    /**
     * 分页查询故障管理列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 故障管理分页列表
     */
    TableDataInfo<TblBreakdownVo> queryPageList(TblBreakdownBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的故障管理列表
     *
     * @param bo 查询条件
     * @return 故障管理列表
     */
    List<TblBreakdownVo> queryList(TblBreakdownBo bo);

    /**
     * 新增故障管理
     *
     * @param bo 故障管理
     * @return 是否新增成功
     */
    Boolean insertByBo(TblBreakdownBo bo);

    /**
     * 修改故障管理
     *
     * @param bo 故障管理
     * @return 是否修改成功
     */
    Boolean updateByBo(TblBreakdownBo bo);

    /**
     * 校验并批量删除故障管理信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
