package org.dromara.service;

import org.dromara.domain.vo.TblRulesNodeVo;
import org.dromara.domain.bo.TblRulesNodeBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 规则节点Service接口
 *
 * @author Lion Li
 * @date 2024-08-15
 */
public interface ITblRulesNodeService {

    /**
     * 查询规则节点
     *
     * @param id 主键
     * @return 规则节点
     */
    TblRulesNodeVo queryById(Long id);

    /**
     * 分页查询规则节点列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 规则节点分页列表
     */
    TableDataInfo<TblRulesNodeVo> queryPageList(TblRulesNodeBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的规则节点列表
     *
     * @param bo 查询条件
     * @return 规则节点列表
     */
    List<TblRulesNodeVo> queryList(TblRulesNodeBo bo);

    /**
     * 新增规则节点
     *
     * @param bo 规则节点
     * @return 是否新增成功
     */
    Boolean insertByBo(TblRulesNodeBo bo);

    /**
     * 修改规则节点
     *
     * @param bo 规则节点
     * @return 是否修改成功
     */
    Boolean updateByBo(TblRulesNodeBo bo);

    /**
     * 校验并批量删除规则节点信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
