package org.dromara.service;

import org.dromara.domain.vo.TblRulesVo;
import org.dromara.domain.bo.TblRulesBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 规则Service接口
 *
 * @author Lion Li
 * @date 2024-08-14
 */
public interface ITblRulesService {

    /**
     * 查询规则
     *
     * @param id 主键
     * @return 规则
     */
    TblRulesVo queryById(Long id);

    /**
     * 分页查询规则列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 规则分页列表
     */
    TableDataInfo<TblRulesVo> queryPageList(TblRulesBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的规则列表
     *
     * @param bo 查询条件
     * @return 规则列表
     */
    List<TblRulesVo> queryList(TblRulesBo bo);

    /**
     * 新增规则
     *
     * @param bo 规则
     * @return 是否新增成功
     */
    Boolean insertByBo(TblRulesBo bo);

    /**
     * 修改规则
     *
     * @param bo 规则
     * @return 是否修改成功
     */
    Boolean updateByBo(TblRulesBo bo) throws Exception;

    /**
     * 校验并批量删除规则信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
