package org.dromara.service;

import org.dromara.domain.vo.TblSensorRecordVo;
import org.dromara.domain.bo.TblSensorRecordBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * 传感器实时数据Service接口
 *
 * @author Lion Li
 * @date 2024-08-11
 */
public interface ITblSensorRecordService {


    void createTable(Map<String, Object> map);

    /**
     * 查询传感器实时数据
     *
     * @param id 主键
     * @return 传感器实时数据
     */
    TblSensorRecordVo queryById(Long id);

    /**
     * 分页查询传感器实时数据列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 传感器实时数据分页列表
     */
    TableDataInfo<TblSensorRecordVo> queryPageList(TblSensorRecordBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的传感器实时数据列表
     *
     * @param bo 查询条件
     * @return 传感器实时数据列表
     */
    List<TblSensorRecordVo> queryList(TblSensorRecordBo bo);

    /**
     * 新增传感器实时数据
     *
     * @param bo 传感器实时数据
     * @return 是否新增成功
     */
    Boolean insertByBo(TblSensorRecordBo bo);

    /**
     * 修改传感器实时数据
     *
     * @param bo 传感器实时数据
     * @return 是否修改成功
     */
    Boolean updateByBo(TblSensorRecordBo bo);

    /**
     * 校验并批量删除传感器实时数据信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
