package org.dromara.service;

import org.dromara.domain.vo.TblSensorVo;
import org.dromara.domain.bo.TblSensorBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 传感器列Service接口
 *
 * @author Lion Li
 * @date 2024-08-11
 */
public interface ITblSensorService {

    /**
     * 查询传感器列
     *
     * @param id 主键
     * @return 传感器列
     */
    TblSensorVo queryById(Long id);

    /**
     * 分页查询传感器列列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 传感器列分页列表
     */
    TableDataInfo<TblSensorVo> queryPageList(TblSensorBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的传感器列列表
     *
     * @param bo 查询条件
     * @return 传感器列列表
     */
    List<TblSensorVo> queryList(TblSensorBo bo);

    /**
     * 新增传感器列
     *
     * @param bo 传感器列
     * @return 是否新增成功
     */
    Boolean insertByBo(TblSensorBo bo);

    /**
     * 修改传感器列
     *
     * @param bo 传感器列
     * @return 是否修改成功
     */
    Boolean updateByBo(TblSensorBo bo);

    /**
     * 校验并批量删除传感器列信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
