package org.dromara.system.service;

import org.dromara.system.domain.vo.TblPageVo;
import org.dromara.system.domain.bo.TblPageBo;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.common.mybatis.core.page.PageQuery;

import java.util.Collection;
import java.util.List;

/**
 * 页面Service接口
 *
 * @author Lion Li
 * @date 2024-07-26
 */
public interface ITblPageService {

    /**
     * 查询页面
     *
     * @param id 主键
     * @return 页面
     */
    TblPageVo queryById(Long id);

    /**
     * 分页查询页面列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 页面分页列表
     */
    TableDataInfo<TblPageVo> queryPageList(TblPageBo bo, PageQuery pageQuery);

    /**
     * 查询符合条件的页面列表
     *
     * @param bo 查询条件
     * @return 页面列表
     */
    List<TblPageVo> queryList(TblPageBo bo);

    /**
     * 新增页面
     *
     * @param bo 页面
     * @return 是否新增成功
     */
    Boolean insertByBo(TblPageBo bo);

    /**
     * 修改页面
     *
     * @param bo 页面
     * @return 是否修改成功
     */
    Boolean updateByBo(TblPageBo bo);

    /**
     * 校验并批量删除页面信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid);
}
